## Load required packages
require("copula")
require("car")
require("rmutil")
require("fitdistrplus") ## Version 1.0-8
require("colorspace")
require("lattice")

## Limits_Data.Rdata contains a vector named "hg" of all relevant effect sizes
## from Weisz, Ng, & Kuppens (2017). 
load("Limits_Data_Pre.RData")  

##load("Limits_Data_Post.RData") 
## Includes post-analysis data, including everything done in
## the supplemental analyses

# Set initial seed
set.seed(123)

##----------------------------------------##
## Exploratory analysis

cor.test(hg, manuals)
cor.test(hg, training)
cor.test(hg, adherence)
cor.test(hg, nsessions)

##----------------------------------------##
## (Figure S2)
## An example of a bivariate distribution
x <- seq( -3, 3, length=25)
y <- seq( -3, 4, length=25)

z <- outer( x, y, function(x,y) dnorm(x,0,0.5)*dgamma(y,1.5, 1))
zl <- c(0,4*max(z))

## Perspective plot 
persp_plot <- persp(x,y,z, theta=125, zlim=zl, box=FALSE, shade=0.75, col="lightblue")

## X and Y grids on the side
for( i in seq(-3,3, by=0.5 ) ) {
  lines(trans3d( c(i,i), c(-3,-3), zl, persp_plot ), col='grey' )
}
for( i in seq(0,zl[2], length=7) ) {
  lines(trans3d( c(-3,3), c(-3,-3), c(i,i), persp_plot ), col='grey' )
}
for( i in seq(-3,4, by=0.5 ) ) {
  lines(trans3d( c(-3,-3), c(i,i), zl, persp_plot ), col='grey' )
}
for( i in seq(0,zl[2], length=7) ) {
  lines(trans3d( c(-3,-3), c(-3,4), c(i,i), persp_plot ), col='grey' )
}

## X distribution
lines(trans3d(seq(-3,3,length=100), -3, .7*dnorm(seq(-3,3,length=100),0,.5), 
                persp_plot), lwd=2, col='black' )
## Y distribution
lines(trans3d(-3, seq(-3,4,length=100), dgamma(seq(-3,4,length=100),1.5,1), 
                persp_plot), lwd=2, col='black' )


##----------------------------------------##
## Descriptives of effect sizes
mean(hg)
median(hg)
var(hg)
hist(hg, breaks=100)

##----------------------------------------##
## Fit a skew laplace (AKA double-exponential) distribution
## to our empirical effect sizes
laplace <- fitdistrplus::fitdist(hg, "skewlaplace", start=list(m=0,s=1,f=1))
m <- laplace$estimate[1]
s <- laplace$estimate[2]
f <- laplace$estimate[3]
## m represents the location parameter, s is the dispersion, and 
## f is the skew. f=1 represents a symmetric distribution
## See ?dskewlaplace for more details

## Visualize double exponential distribution fit
hist(hg, prob=T, xlim=c(-3, 5), ylim=c(0,1.5), breaks=80, main = "",
     xlab= "Effect Size of Therapy")
## Double-exponential distribution
lines(sort(hg) , y = dskewlaplace(sort(hg),m = m , s = s,f = f), col = "black" , lty = 1 , lwd = 2)
## Kernel smoother fit
lines(density(hg), lwd=2, col="black", lty=2)
legend(x=1.8,y=1.5, legend=c("Distribution", "Kernel Smoother"), lty=c(1,2))

##----------------------------------------##
## Fit/plot a copula for demonstration
## purposes.

## Coloring
copColor <- terrain_hcl(10000, h=c(130,-10))

## Dependence=0.2 (example)
## Fit the copula
cop0 <- mvdc(normalCopula(0.2), margins = c("norm", "skewlaplace"),
             paramMargins = list(list(mean = 0, sd = 1),
                                 list(m=m, s=s, f=f)))
## Get the output from the plot
out0 <- persp(cop0, dMvdc, xlim=c(-4, 4), ylim=c(-0.5, 2.5), xlab="Quality of therapy",
             ylab="Effect Size", zlab= "Probability", mgp=c(3,3,3),
            n.grid=100, theta=30)

## Additional coloring parameters
z <- out0$z
nrz<-nrow(z)
ncz<-ncol(z)
zfacet<-z[-1,-1]+z[-1,-ncz]+z[-nrz,-1]+z[-nrz,-ncz]
facetcol<-cut(zfacet,10000)

## Plot
out1 <- persp(cop0, dMvdc, xlim=c(-4, 4), ylim=c(-0.5, 2.5), xlab="Quality of therapy",
              ylab="Effect Size", zlab= "Probability", mgp=c(3,3,3),
              n.grid=100, theta=80)
persp(cop0, dMvdc, xlim=c(-4, 4), ylim=c(-0.5, 2.5), xlab="Quality of therapy",
      ylab="Effect Size", zlab= "Probability", mgp=c(3,3,3),
      col=copColor[facetcol], n.grid=100, theta=80)
## effect size distribution
lines(trans3d(y=seq(-0.5,2.5,length=100),x=-4, z=.4*dskewlaplace(seq(-0.5,2.5,length=100),m = m , s = s,f = f), 
              out1$persp), lwd=2, col='grey' )
## Y distribution
lines(trans3d(y=2.5, x=seq(-3,4,length=100), z=.7*dnorm(seq(-3,4,length=100),0,1), 
              out1$persp), lwd=2, col='grey' )
## Upper limit
lines(trans3d(y=c(0.851,0.851), x=c(-4,4), z=c(0,0), out1$persp ), col='grey' )

## Contour version
## This is essentially a "top down" view of the same distribution
cols <- rev(heat_hcl(33, c = c(80, 30), l = c(30, 90)))
contourplot2(cop0, FUN = dMvdc, xlim=c(-4, 4), ylim=c(-0.8, 2.5), ylab = "Effect Size", xlab = "Quality of Therapy", 
             col.regions = cols, cuts = 16, n.grid = c(200,200), at = seq(0, 0.5, 0.05),
             panel=function(...){
               panel.contourplot(...)
               panel.abline(h = 0.851, col = "black")
             }) 


## Create function to create copula, draw a sample of 1000
## from the copula, fit a line through the sample, and find the
## value of effect size at the 99.9th percentile of quality
upperlimit <- function(x){
  cop1 <- mvdc(normalCopula(x), margins = c("norm", "skewlaplace"),
               paramMargins = list(list(mean = 0, sd = 1),
                                   list(m=m, s=s, f=f)))
  x1.samp <- as.data.frame(rMvdc(1000, cop1))
  fit1 <- lm(V2~V1, data=x1.samp)
  t <- as.numeric(fit1$coefficients[1] + 3.291*fit1$coefficients[2])
  return(t)
}

##----------------------------------------##
## Let's make a table of correlation --> upper estimate
## We'll have 100s samples for each value of correlation.  
## based on samples of 1000 from copula models with
## correlation values from 0.01 to 1

## 0.01, 0.02, etc.
myseq <- seq(0.00, 1, 0.01)
## Repeat each one 100 times
numTab <- sort(rep(myseq, 100))
## Same process as before
simulated_ulTab<-sapply(numTab, upperlimit)
## Visualize
hist(simulated_ulTab)
## Make a dataframe including the correlation values, 
## to make extracting medians easier
TabDF <- cbind(numTab, simulated_ulTab)
## Find the median value for each value of correlation
medians2<-vector()
for(i in 1:length(myseq)){
  medians2[i] <- median(TabDF[,2][TabDF[,1]==myseq[i]])
}
## Make a final table
table1 <- cbind(myseq, medians2)
table1

##----------------------------------------##
## Let's simulate 10000 estimated upper limits
## based on samples of 1000 from copulas with
## random correlation values bounded
## by 0.1 and 0.3

set.seed(123)
num1.5 <- runif(10000, min=0.1, max=0.3)
simulated_ul1.5<-sapply(num1.5, upperlimit)

## Visualize
hist(simulated_ul1.5)

## Find expected value
median(simulated_ul1.5)
## Confidence interval (given bounded correlations)
quantile(simulated_ul1.5, c(.05, .95))

## Interpretation: if we could acheive a 99.9% quality of therapy, we would reasonably expect
## an effect size of about 
median(simulated_ul1.5)
## with 95% confidence interval:
quantile(simulated_ul1.5, c(.05, .95))


