##----------------------------------------##
## SUPPLEMENTAL MATERIALS
require("copula")
require("car")
require("rmutil")
require("fitdistrplus") # Version 1.0-8
require("sn")
require("normalp")

## load data and set seed
load("Limits_Data_Post2.RData") ## For post-analysis data
## This loads all data, including the finished models
set.seed(123)

##----------------------------------------##
## Make an animation of the copula

## Coloring
jet.colors <-  colorRampPalette(c("midnightblue","blue",
                                  "cyan","green", "yellow","orange","red", "darkred"))
copColor <- jet.colors(10000)

## Function to plot a copula given dependence value "x"
createcop <- function(x, theta=-30) {
  copula1 <- mvdc(normalCopula(x), margins = c("norm", "skewlaplace"),
                  paramMargins = list(list(mean = 0, sd = 1),
                                      list(m=m, s=s, f=f)))
  out <- persp(copula1, dMvdc, xlim=c(-4, 4), ylim=c(-1, 4), zlim=c(0,1.2), 
        col="lightblue", xlab="Quality of Therapy", ylab="Effect Size", zlab="Frequency",
        cex.lab=.8, theta=theta, phi=30, r=1, n.grid=100)
  z <- out$z
  nrz<-nrow(z)
  ncz<-ncol(z)
  zfacet<-z[-1,-1]+z[-1,-ncz]+z[-nrz,-1]+z[-nrz,-ncz]
  facetcol<-cut(zfacet,10000)
  persp(copula1, dMvdc, xlim=c(-4, 4), ylim=c(-1, 4), zlim=c(0,1.2), 
       xlab="Quality of Therapy", ylab="Effect Size", zlab="Frequency",
        cex.lab=.8, theta=theta, phi=30, r=1, col=copColor[facetcol], n.grid=100)
  legend("topright", paste("r=", x), bty="n")
}
## check to see if that works:
createcop(.2)

## Time to animate
myVec <- NULL
## Note: values greater than 0.92 do not plot well
myVec <- c((1:92*.01), (92:1*.01))
nsteps <- length(myVec)

## Saving each frame as a PNG
## Then make a GIF using any GIF software
dir.create("./Dependence/")
setwd("~/Dependence")
for(i in 1:184) {
  name <- paste("image",i,".png")
  png(filename=name, width=1600, height=1600)
  createcop(myVec[i])
  dev.off()
}
## For the "full range" animation use all frames
## For the "limited range" animation use frames 10-30, 155-175

## Let's do that from a different viewpoint
dir.create("~/Dependence2/")
setwd("~/Dependence2")
for(i in 1:184) {
  name <- paste("image",i,".png")
  png(filename=name, width=1600, height=1600)
  createcop(myVec[i], theta=60)
  dev.off()
}

## Also make some animations to rotate around theta
dir.create("~/Rotation/")
setwd("~/Rotation")
myVec2 <- 1:360
for(i in myVec2) {
  name <- paste("rotate_image",i,".png")
  png(filename=name, width=1600, height=1600)
  createcop(x=0.2, theta=myVec2[i])
  dev.off()
}

## Make an animation of the contour plots
createcontour <- function(x) {
  copula1 <- mvdc(normalCopula(x), margins = c("norm", "skewlaplace"),
                  paramMargins = list(list(mean = 0, sd = 1),
                                      list(m=m, s=s, f=f)))
  contour(copula1, dMvdc, xlim=c(-3, 3), ylim=c(-1, 2.5),
          col="blue", xlab="Quality of Therapy", ylab="Effect Size", nlevels=10,
          cex.lab=.8)
  legend("topleft", paste("r=", x), bty="n")
}

for(i in 1:184) {
  name <- paste("imagecontour", i, ".png")
  png(filename=name)
  createcontour(myVec[i])
  dev.off()
}

## .png files can then be animated with imagemagick

##----------------------------------------##
## Sensitivity analysis

## In the original distribution, 0s were inputted for all studies
## where authors indicated that the effect size was non-significant,
## without reporting the actual effect sizes

## Remove 0s
hg2 <- hg[hg!=0]

laplace2 <- fitdistrplus::fitdist(hg2, "skewlaplace", start=list(m=0,s=1,f=1))
m2 <- laplace2$estimate[1]
s2 <- laplace2$estimate[2]
f2 <- laplace2$estimate[3]

snormalfit <- fitdistrplus::fitdist(hg2, "sn", start=list(xi=0, omega=1, alpha=0, tau=0), 
                                    lower=c(xi=-Inf, omega=0, alpha=-Inf, tau=-0.00001),
                                    upper=c(xi=Inf, omega=Inf, alpha=Inf, tau=0.00001))

hist(hg2, prob=T, xlim=c(-3, 5), ylim=c(0,1.5), breaks=80, main = "Fit to Histogram",
     xlab= "Effect Size of Therapy")
lines(sort(hg2) , y = dskewlaplace(sort(hg2),m = m2 , s = s2,f = f2), col = "blue" , lty = 2 , lwd = 2)
lines(sort(hg2) , y = dsn(sort(hg2), xi=snormalfit$estimate[1], omega=snormalfit$estimate[2],
                          alpha=snormalfit$estimate[3], tau=snormalfit$estimate[4]),
      col = "red", lty = 2 , lwd = 2)

## 10,000 random correlation values between 0.1 and 0.3
num_sens <- runif(10000, min=0.1, max=0.3)

## Create function
upperlimit2 <- function(x){
  cop1 <- mvdc(normalCopula(x), margins = c("norm", "skewlaplace"),
               paramMargins = list(list(mean = 0, sd = 1),
                                   list(m=m2, s=s2, f=f2)))
  x1.samp <- as.data.frame(rMvdc(1000, cop1))
  fit1 <- lm(V2~V1, data=x1.samp)
  t <- as.numeric(fit1$coefficients[1] + 3.291*fit1$coefficients[2])
  return(t)
}

## Apply that function to all 10000 correlation values
simulated_ul2<-sapply(num_sens, upperlimit2)

## Visualize
hist(simulated_ul2)

median_sens <- median(simulated_ul2)

## Generate confidence interval
conf_sens <- quantile(simulated_ul2, c(.05, .95))

##----------------------------------------##
## OTHER DISTRIBUTIONS
##----------------------------------------##
## ALTERNATIVES TO SKEW-LAPLACE FIT (EFFECT SIZES)
## Normal distribution (allowed to vary by location) (1a)

normalfit <- fitdistrplus::fitdist(hg, "sn", start=list(xi=0, omega=1, alpha=0, tau=0), 
                                   lower=c(xi=-Inf, omega=0, alpha=-0.0001, tau=-0.0001),
                                   upper=c(xi=Inf, omega=Inf, alpha=0.0001, tau=0.0001))
hist(hg, prob=T, xlim=c(-3, 5), ylim=c(0,1.5), breaks=80, main = "Fit to Histogram",
     xlab= "Effect Size of Therapy")
lines(sort(hg) , y = dsn(sort(hg), xi=normalfit$estimate[1], omega=normalfit$estimate[2],
                         alpha=normalfit$estimate[3], tau=normalfit$estimate[4]),
      col = "blue", lty = 2 , lwd = 2)

num1a <- runif(10000, min=0.1, max=0.3)
upperlimit1a <- function(x){
  cop1a <- mvdc(normalCopula(x), margins = c("norm", "sn"),
                paramMargins = list(list(mean = 0, sd = 1),
                                    list(xi=normalfit$estimate[1], omega=normalfit$estimate[2],
                                         alpha=normalfit$estimate[3], tau=normalfit$estimate[4])))
  x1a.samp <- as.data.frame(rMvdc(1000, cop1a))
  fit1a <- lm(V2~V1, data=x1a.samp)
  t1a <- as.numeric(fit1a$coefficients[1] + 3.291*fit1a$coefficients[2])
  return(t1a)
}
simulated_ul1a<-sapply(num1a, upperlimit1a)
med1a <- median(simulated_ul1a)
conf1a <- quantile(simulated_ul1a, c(.05, .95))


## Gamma distribution (2a)
# note: distribution must be initially shifted so minimum>0
gammafit <- fitdistrplus::fitdist((hg-(min(hg)-0.0001)), "gamma", start=list(shape=10, rate=1))
hist((hg-(min(hg)-0.0001)), prob=T, xlim=c(-0, 8), ylim=c(0,1.5), breaks=80, main = "Fit to Histogram",
     xlab= "Effect Size of Therapy")
lines(sort((hg-(min(hg)-0.0001))) , y = dgamma(sort((hg-(min(hg)-0.0001))), shape=gammafit$estimate[1], rate=gammafit$estimate[2]),
      col = "blue", lty = 2 , lwd = 2)

num2a <- runif(10000, min=0.1, max=0.3)
upperlimit2a <- function(x){
  cop2a <- mvdc(normalCopula(x), margins = c("norm", "gamma"),
                paramMargins = list(list(mean = 0, sd = 1),
                                    list(shape=gammafit$estimate[1], rate=gammafit$estimate[2])))
  x2a.samp <- as.data.frame(rMvdc(1000, cop2a))
  fit2a <- lm(V2~V1, data=x2a.samp)
  t2a <- as.numeric(fit2a$coefficients[1] + 3.291*fit2a$coefficients[2])
  return(t2a)
}
simulated_ul2a<-sapply(num2a, upperlimit2a)
med2a <- median(simulated_ul2a) + (min(hg)-0.0001)
conf2a <- quantile(simulated_ul2a, c(.05, .95)) + (min(hg)-0.0001)

## Power normal (dnormp) (3a) 
# note: solution is equivalent to a skew-laplace function

normpfit <- fitdistrplus::fitdist(hg, "normp", start=list(mu=0, sigmap=1, p=2),
                                  lower=c(-5,-5, 1), upper=c(5,5,Inf))
hist(hg, prob=T, xlim=c(-4, 9), ylim=c(0,2), breaks=80, main = "Fit to Histogram",
     xlab= "Effect Size of Therapy")
lines(sort(hg) , y = dnormp(sort(hg), mu=normpfit$estimate[1], sigmap=normpfit$estimate[2], p=normpfit$estimate[3]),
      col = "blue", lty = 2 , lwd = 2)

num3a <- runif(10000, min=0.1, max=0.3)
upperlimit3a <- function(x){
  cop3a <- mvdc(normalCopula(x), margins = c("norm", "normp"),
                paramMargins = list(list(mean = 0, sd = 1),
                                    list(mu=normpfit$estimate[1], sigmap=normpfit$estimate[2], p=normpfit$estimate[3])))
  x3a.samp <- as.data.frame(rMvdc(1000, cop3a))
  fit3a <- lm(V2~V1, data=x3a.samp)
  t3a <- as.numeric(fit3a$coefficients[1] + 3.291*fit3a$coefficients[2])
  return(t3a)
}
simulated_ul3a<-sapply(num3a, upperlimit3a)
med3a <- median(simulated_ul3a) 
conf3a <- quantile(simulated_ul3a, c(.05, .95))

## Cauchy (dcauchy) (4a)

cauchyfit <- fitdistrplus::fitdist(hg, "cauchy", start=list(location=0, scale=1),
                                   lower=c(-Inf,0))
hist(hg, prob=T, xlim=c(-3, 5), ylim=c(0,1.5), breaks=80, main = "Fit to Histogram",
     xlab= "Effect Size of Therapy")
lines(sort(hg) , y = dcauchy(sort(hg), location=cauchyfit$estimate[1], scale=cauchyfit$estimate[2]),
      col = "blue", lty = 2 , lwd = 2)

num4a <- runif(10000, min=0.1, max=0.3)
upperlimit4a <- function(x){
  cop4a <- mvdc(normalCopula(x), margins = c("norm", "cauchy"),
                paramMargins = list(list(mean = 0, sd = 1),
                                    list(location=cauchyfit$estimate[1], scale=cauchyfit$estimate[2])))
  x4a.samp <- as.data.frame(rMvdc(1000, cop4a))
  fit4a <- lm(V2~V1, data=x4a.samp)
  t4a <- as.numeric(fit4a$coefficients[1] + 3.291*fit4a$coefficients[2])
  return(t4a)
}
simulated_ul4a<-sapply(num4a, upperlimit4a)
med4a <- median(simulated_ul4a) 
conf4a <- quantile(simulated_ul4a, c(.05, .95))

## Log-normal (dlnorm) (5a)
# note: distribution must be initially shifted so minimum>0

lnormfit <- fitdistrplus::fitdist((hg-(min(hg)-0.0001)), "lnorm", start=list(meanlog=0, sdlog=1),
                                  lower=c(-Inf,0))
hist(hg-(min(hg)-0.0001), prob=T, xlim=c(0, 8), ylim=c(0,1.5), breaks=80, main = "Fit to Histogram",
     xlab= "Effect Size of Therapy")
lines(sort(hg-(min(hg)-0.0001)) , y = dlnorm(sort(hg-(min(hg)-0.0001)), meanlog=lnormfit$estimate[1], sdlog=lnormfit$estimate[2]),
      col = "blue", lty = 2 , lwd = 2)

num5a <- runif(10000, min=0.1, max=0.3)
upperlimit5a <- function(x){
  cop5a <- mvdc(normalCopula(x), margins = c("norm", "lnorm"),
                paramMargins = list(list(mean = 0, sd = 1),
                                    list(meanlog=lnormfit$estimate[1], sdlog=lnormfit$estimate[2])))
  x5a.samp <- as.data.frame(rMvdc(1000, cop5a))
  fit5a <- lm(V2~V1, data=x5a.samp)
  t5a <- as.numeric(fit5a$coefficients[1] + 3.291*fit5a$coefficients[2])
  return(t5a)
}
simulated_ul5a<-sapply(num5a, upperlimit5a)
med5a <- median(simulated_ul5a) + (min(hg)-0.0001)
conf5a <- quantile(simulated_ul5a, c(.05, .95)) + (min(hg)-0.0001)

#save.image("Limits_Data_AlternativeDistributions.RData")

##----------------------------------------##
## ALTERNATIVES TO NORMAL DISTRIBUTION (QUALITY)

## Normal distribution with left skew (1b)
x<-seq(-4,4,length=2000)
y<-dsn(x,xi=0, omega=1, alpha=-2, tau=0)
plot(x,y, type="l", lwd=2)

num1b <- runif(10000, min=0.1, max=0.3)
upperlimit1b <- function(x){
  cop1b <- mvdc(normalCopula(x), margins = c("sn", "skewlaplace"),
                paramMargins = list(list(xi=0, omega=1, alpha=-2, tau=0),
                                    list(m=m, s=s, f=f)))
  x1b.samp <- as.data.frame(rMvdc(1000, cop1b))
  fit1b <- lm(V2~V1, data=x1b.samp)
  t1b <- as.numeric(fit1b$coefficients[1] + 3.291*fit1b$coefficients[2])
  return(t1b)
}
simulated_ul1b<-sapply(num1b, upperlimit1b)
med1b <- median(simulated_ul1b) 
conf1b <- quantile(simulated_ul1b, c(.05, .95)) 

## Normal distribution with right skew (2b)
y<-dsn(x,xi=0, omega=1, alpha=2, tau=0)
plot(x,y, type="l", lwd=2)

num2b <- runif(10000, min=0.1, max=0.3)
upperlimit2b <- function(x){
  cop2b <- mvdc(normalCopula(x), margins = c("sn", "skewlaplace"),
                paramMargins = list(list(xi=0, omega=1, alpha=2, tau=0),
                                    list(m=m, s=s, f=f)))
  x2b.samp <- as.data.frame(rMvdc(1000, cop2b))
  fit2b <- lm(V2~V1, data=x2b.samp)
  t2b <- as.numeric(fit2b$coefficients[1] + 3.291*fit2b$coefficients[2])
  return(t2b)
}
simulated_ul2b<-sapply(num2b, upperlimit2b)
med2b <- median(simulated_ul2b) 
conf2b <- quantile(simulated_ul2b, c(.05, .95)) 

## Laplace distribution (3b)
y<-dskewlaplace(x,m=0, s=1, f=1)
plot(x,y, type="l", lwd=2)

num3b <- runif(10000, min=0.1, max=0.3)
upperlimit3b <- function(x){
  cop3b <- mvdc(normalCopula(x), margins = c("skewlaplace", "skewlaplace"),
                paramMargins = list(list(m=0, s=1, f=1),
                                    list(m=m, s=s, f=f)))
  x3b.samp <- as.data.frame(rMvdc(1000, cop3b))
  fit3b <- lm(V2~V1, data=x3b.samp)
  t3b <- as.numeric(fit3b$coefficients[1] + 3.291*fit3b$coefficients[2])
  return(t3b)
}
simulated_ul3b<-sapply(num3b, upperlimit3b)
med3b <- median(simulated_ul3b) 
conf3b <- quantile(simulated_ul3b, c(.05, .95)) 

## Laplace distribution (left skew) (4b)
y<-dskewlaplace(x,m=0, s=1, f=1.333)
plot(x,y, type="l", lwd=2)

num4b <- runif(10000, min=0.1, max=0.3)
upperlimit4b <- function(x){
  cop4b <- mvdc(normalCopula(x), margins = c("skewlaplace", "skewlaplace"),
                paramMargins = list(list(m=0, s=1, f=1.333),
                                    list(m=m, s=s, f=f)))
  x4b.samp <- as.data.frame(rMvdc(1000, cop4b))
  fit4b <- lm(V2~V1, data=x4b.samp)
  t4b <- as.numeric(fit4b$coefficients[1] + 3.291*fit4b$coefficients[2])
  return(t4b)
}
simulated_ul4b<-sapply(num4b, upperlimit4b)
med4b <- median(simulated_ul4b) 
conf4b <- quantile(simulated_ul4b, c(.05, .95)) 

## Laplace distribution (right skew) (5b)
y<-dskewlaplace(x,m=0, s=1, f=0.75)
plot(x,y, type="l", lwd=2)

num5b <- runif(10000, min=0.1, max=0.3)
upperlimit5b <- function(x){
  cop5b <- mvdc(normalCopula(x), margins = c("skewlaplace", "skewlaplace"),
                paramMargins = list(list(m=0, s=1, f=.75),
                                    list(m=m, s=s, f=f)))
  x5b.samp <- as.data.frame(rMvdc(1000, cop5b))
  fit5b <- lm(V2~V1, data=x5b.samp)
  t5b <- as.numeric(fit5b$coefficients[1] + 3.291*fit5b$coefficients[2])
  return(t5b)
}
simulated_ul5b<-sapply(num5b, upperlimit5b)
med5b <- median(simulated_ul5b) 
conf5b <- quantile(simulated_ul5b, c(.05, .95)) 

## Cauchy distribution (6b)
y<-dcauchy(x,location=0, scale=1)
plot(x,y, type="l", lwd=2)

num6b <- runif(10000, min=0.1, max=0.3)
upperlimit6b <- function(x){
  cop6b <- mvdc(normalCopula(x), margins = c("cauchy", "skewlaplace"),
                paramMargins = list(list(location=0, scale=1),
                                    list(m=m, s=s, f=f)))
  x6b.samp <- as.data.frame(rMvdc(1000, cop6b))
  fit6b <- lm(V2~V1, data=x6b.samp)
  t6b <- as.numeric(fit6b$coefficients[1] + 3.291*fit6b$coefficients[2])
  return(t6b)
}
simulated_ul6b<-sapply(num6b, upperlimit6b)
med6b <- median(simulated_ul6b) 
conf6b <- quantile(simulated_ul6b, c(.05, .95)) 

## Create table

quality_dist <- c(rep("Normal", 5), "Normal (Left skew)", "Normal (Right skew)", "Laplace", "Laplace (Left skew)", "Laplace (Right skew)", "Cauchy")
es_dist <- c("Normal", "Gamma", "Power normal", "Cauchy", "Log-normal", rep("Skew-Laplace", 6))
medians <- c(med1a, med2a, med3a, med4a, med5a, med1b, med2b, med3b, med4b, med5b, med6b)
conflow <- c(conf1a[1], conf2a[1], conf3a[1], conf4a[1], conf5a[1], conf1b[1], conf2b[1], conf3b[1], conf4b[1], conf5b[1], conf6b[1])
confhigh <- c(conf1a[2], conf2a[2], conf3a[2], conf4a[2], conf5a[2], conf1b[2], conf2b[2], conf3b[2], conf4b[2], conf5b[2], conf6b[2])
table1 <- cbind(quality_dist, es_dist, medians, conflow, confhigh)

write.csv(table1, file="Alternative_Distributions.csv")

median(medians[6:11])

## Did Laplace have the best fit?
## Laplace has the lowest BIC at 10556

summary(laplace)
summary(normalfit)
summary(gammafit)
summary(normpfit)
summary(cauchyfit)
summary(lnormfit)

##----------------------------------------##
## Creating a histogram to visualize the table of reviewed articles:

tabler2s <-c(0.00118,0.00092,0.00400,0.00490,0.04000,0.00250,0.00000,0.00283,0.02132,0.06605,
             0.08009,0.04752,0.05063,0.00490,0.06864,0.00137,0.04537,0.00624,0.03960,0.00023,
             0.00640,0.07840,0.10890,0.05760,0,0.22090,0.08570,0.00000,0.00000,0.00010,0.40967,
             0.07311,0)

hist(tabler2s, breaks=20, col="lightblue",
     xlim=c(0, 0.55), ylim=c(0,20),
     main="Relationship Between Quality and Outcome: Effect Sizes in the Literature",
     ylab="Frequency",xlab=expression(paste("Effect size (","r"^"2", ")")))
